/*
 * steve diverdi
 * cs 125
 * project 3
 *
 * sockets wrappers
 */

// INCLUDES-------------------------------------------------------------


#include "sockets.hh"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream.h>
#include <strings.h>
#include <inttypes.h>				// for uint16_t

#include <sys/types.h>				// datatypes
#include <sys/socket.h>				// socket calls

#include <netinet/in.h>                         // internet protocol
#include <netdb.h>                              // for hostent

#include <errno.h>


// ---------------------------------------------------------------------


/*
 * Create a socket and bind it to port.  Use 0 if a random port is okay.
 */
int createSocket (
    int		port)
{
    struct sockaddr_in addr;			// socket address
    bzero((char*) &addr, sizeof(addr));
    addr.sin_family = AF_INET;			// internet
    addr.sin_port = htons(port);		// network byte order
    addr.sin_addr.s_addr = htonl(INADDR_ANY);	// any source

    int s;					// create the socket
    if ((s = socket(PF_INET, SOCKET_TYPE, 0)) < 0) 
    {
        cerr << "createSocket: socket" << endl;
        exit(1);
    }

    int one = 1;				// set socket reusable
    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char*) &one, 
                   sizeof(one)) < 0) 
    {
        cerr << "createSocket: setsockopt" << endl;
        exit(1);
    }

						// bind socket to address
    if (bind(s, (struct sockaddr*) &addr, sizeof(struct sockaddr))) 
    {
        cerr << "createSocket: bind" << endl;
        exit(1);
    }

    return s;
}


/*
 * Create a socket to listen for connections on.  Takes the port to
 * listen to, and returns the identifier of the socket created.
 */
int createServerSocket (
    int		port) 
{
    int s = createSocket(port);
						// set # connections
    if (listen(s, MAX_PENDING)) 
    {
        cerr << "createServerSocket: listen" << endl;
        exit(1);
    }

    return s;
}


/*
 * Wait for a connection on an existing socket.  Takes the socket id as
 * an argument, and returns the identifier of the connection.
 */
int acceptConnection (
    int		server)
{
    struct sockaddr_in addr;			// connection address
    int addrlen = sizeof(addr);

    int connection;				// accept the connection
    if ((connection = accept(server, (struct sockaddr*) &addr, &addrlen)) < 0)
    {
        cerr << "acceptConnection: accept" << endl;
        exit(1);
    }

    return connection;
}


// ---------------------------------------------------------------------


/*
 * Open a connection to a host / port.  Takes the host and port as input
 * and returns the identifier of the connection that was created.
 */
int connectToServer (
    char*	hostname,
    int		port)
{
    struct sockaddr_in sin;
    createAddr(&sin, hostname, port);

    int connection;				// make the socket
    if ((connection = socket(AF_INET, SOCKET_TYPE, 0)) < 0)
    {
        cerr << "connectToServer: socket" << endl;
        exit(1);
    }

						// connect to server
    if (connect(connection, (struct sockaddr*) &sin, sizeof(sin)) < 0)
    {
        cerr << "connectToServer: connect" << endl;
        exit(1);
    }

    return connection;
}


// ---------------------------------------------------------------------



/*
 * format an address. takes as arguments the address to format, the
 * hostname, and the port number.
 */
void createAddr (
    struct sockaddr_in*	addr,
    char*		hostname,
    int			port)
{
    // get the host ip
    struct hostent* host = gethostbyname(hostname);
    if (!host)
    {
        cerr << "createAddr: gethostbyname" << endl;
        exit(1);
    }

    // initialize the address to zeroes
    bzero(addr, sizeof(addr));

    // put the ip and port into the address
    bcopy(host->h_addr, (char*) &addr->sin_addr, host->h_length);
    addr->sin_family = AF_INET;
    addr->sin_port = htons(port);
}
